#!/usr/bin/perl
# ts2matlab.pl
# Written by Scott T. Marshall
# 08/09/2014
#
# This script grabs fault mesh data from a gocad t-surf file and prints files ending in .xyz (vertices) 
# and .tri (tesselation pattern). The .tri and .xyz files can be rendered easily with MATLAB.
# The MATLAB script plotMesh.m will visualize the fault mesh.
#
#------------------------------------------------------------------------------------------------------------------#

#Should I convert the data from meters to km? 0=no 1=yes
$convertKM=0;

#Pull file name from the command line.
($tsFile) = @ARGV;

#Make sure that the correct number of arguments have been entered on the command line.
if (@ARGV != 1){
	print "Usage: ts2matlab.pl file.ts \n"; 
	exit;
}#end if


#------------------------------------------------------------------------------------------------------------------#
#-----  READ THE GOCAD TS FILE AND PRINT THE NEW MATLAB FILES  ----------------------------------------------------#
#------------------------------------------------------------------------------------------------------------------#
#Open the ts file for reading
open(TS,$tsFile) or die("Error! Cannot Open $tsFile");

#Make counters for keeping track of elements and vertices
$countV=0;
$countT=0;
$countF=0;
$countAllV=0;
$countAllT=0;

#Print messages
print "Reading $tsFile...\n\n";
print "-------------------------------------------------------\n";
print "FaultName  NumElts  NumVertices\n";
print "-------------------------------------------------------\n";
#Read the ts file line by line
while (<TS>){
	chomp;
	#Store each line in an array separated by spaces
	@data=split(" ", $_);
	
	if($data[0] =~ "name:"){
		#Increment the fault counter
		$countF++;
		#remove the name: string and store the fault name in $name
		$data[0] =~ s/name://;
		$name=$data[0];
		#open new files for writing the matlab-friendly files
		open(TRI,">$name.tri");
		open(XYZ,">$name.xyz");
		print TRI "\%Triangulation Pattern\n";
		if($convertKM==1){
			print XYZ "\%X(km) Y(km) Z(km)\n";
		}#end if
		else {
			print XYZ "\%X(m) Y(m) Z(m)\n";
		}#end else
	
	}#end if
	
	#If the first word on the line is VRTX, print the vertices to the xyz file
	elsif($data[0] eq "VRTX"){
		#Increment the VRTX counters
		$countV++;
		$countAllV++;
		#Convert the data, if necessary
		if($convertKM==1){
			$data[2]/=1000;
			$data[3]/=1000;
			$data[4]/=1000;
		}#end if
		
		#Print the data to the XYZ file
		print XYZ "$data[2] $data[3] $data[4]\n";
      }#end elsif
	
	#If the first word on the line is TRGL, print the tesselation pattern
	elsif($data[0] eq "TRGL"){
		#Increment the VRTX counters
		$countT++;
		$countAllT++;
		#Print the data to the TRI file
		print TRI "$data[1] $data[2] $data[3]\n";
      }#end elsif
	
	#If the first word on the line is END, then close the opened files, so the next fault can make new files.
	elsif($data[0] eq "END"){
		close(TRI) or die("Error! Cannot close $name.tri");
		close(XYZ) or die("Error! Cannot close $name.xyz");
		
		#print out the stats for each fault
		print "$name $countT $countV\n";
		
		#Reset the counters for the next fault
		$countV=0;
		$countT=0;
		
	}#end elsif
	
}#end while

#Print out total file stats
print "-------------------------------------------------------\n";
print "GoCAD File:     $tsFile\n";
print "Total Faults:   $countF\n";
print "Total Elts:     $countAllT\n";
print "Total Vertices: $countAllV\n";
print "-------------------------------------------------------\n";
print "Finished!\n";

